# frozen_string_literal: true

module VG
  module ServiceObject

    module ClassSingletonMethods
      def call(...)
        callable.call(...)
      end
    
      def callable
        self.new
      end

      def as_so(method_name = nil)
        ::VG::SO::GenerateIncludableModule.call(self.name, as: method_name)
      end
    end

    module ModuleSingletonMethods
      def call(...)
        callable.call(...)
      end
    
      def callable
        self::Action.new
      end

      def as_so(method_name = nil)
        ::VG::SO::GenerateIncludableModule.call(self.name, as: method_name)
      end
    end

    def self.included(so)
      methods = so.is_a?(Class) ? ClassSingletonMethods : ModuleSingletonMethods
      return if so.singleton_class.included_modules.include?(methods)
      so.extend methods
      
      so.singleton_class.instance_eval do
        define_method(:included) { |_base| raise "Can't be included. Use #as_so method instead" }
      end if so.instance_of?(Module)
    end

  end # ... ServiceObject
end # ... VG
